UIHotUpdate = class("UIHotUpdate", function()
	return CCLayer:create()
end)

UIHotUpdate.__index = UIHotUpdate

local VerPropList= {}
local TotalPkgSize = 0	--所有更新文件的总大小，单位M

function UIHotUpdate:ctor()
	self.mCurVersion = "0.00"
	
	-- self:setTouchEnabled(true)
	
	local function onNodeEvent(event)
        if event.name == "enter" then
			
			self:init()
			
        elseif event.name == "exit" then
			
			CCNotificationCenter:sharedNotificationCenter():removeAllObservers(self)
        end
    end

    self:addNodeEventListener(NODE_EVENT, onNodeEvent)
	
	return self
end

function UIHotUpdate:init()
	local notificationCenter = CCNotificationCenter:sharedNotificationCenter()
	--请求更新配置数据观察者
	notificationCenter:registerScriptObserver(self, function(msgId, obj)
		self:uiCheckUpdate(msgId, obj)
	end, "NotifyRequestSucess")
	
	--下载文件成功观察者
	notificationCenter:registerScriptObserver(self, function(msgId, obj)
		local tVerInfo = VerPropList[self.mCurVersion]
		
		if tVerInfo then
			--更新当前版本号
			self.mCurVersion = tVerInfo["version_next"]
			
			--保存版本号
			self:saveVersion()
			
			tVerInfo = VerPropList[self.mCurVersion]
			
			if tVerInfo then
				--再次检查是否需要更新
				self:uiCheckUpdate("", CCBool:create(false))
			else
				self:uiUpdateFinish(true)
			end
		else
			--重启游戏
			self:uiUpdateFinish(true)
		end
		
	end, "NotifyDownloadSuccess")
	
	--下载出错
	notificationCenter:registerScriptObserver(self, function(msgId, obj)
		-- print("mir_debug:========>download err>>", obj:getValue())
		if self.downloader then --下载出错，重新下载该文件
			local tVerInfo = VerPropList[self.mCurVersion]
			self.downloader:startUpdate(tVerInfo["version_file"])
		end
	end, "NotifyDownloadError")
	
	local isDelete = false
	
	--初始化版本信息
	self:loadVersion()
	
	self.mCurVersion = string.trim(self.mCurVersion)--清楚两端空格
	if self.mCurVersion==nil or self.mCurVersion=="" then
		--最新版本号不存在，获取客户端配置的最低版本号
		self.mCurVersion = GetVersion()
		
		self:saveVersion()
	else
		local ver = GetVersion()	--客户端配置的最低版本号
		if tonumber(self.mCurVersion)<tonumber(ver) then
			self.mCurVersion = ver
			self:saveVersion()
			isDelete = true
		end
	end
	
	--下载器
	self.downloader = Downloader:create(isDelete)
	self:addChild(self.downloader)
	
	print("===============cur_version=", self.mCurVersion)
	-- self.mCurVersion = "1.00"
	
	--请求更新配置
	RequestUpdateInfo()
	
end

--[[
	显示更新UI
]]
function UIHotUpdate:uiShowUpdate()
	TotalPkgSize = 0
	local function calPkgSize(_ver)
		local tVerInfo = VerPropList[_ver]
		if tVerInfo then
			TotalPkgSize = TotalPkgSize + tVerInfo["version_size"]
			calPkgSize(tVerInfo["version_next"])
		end
	end
	
	calPkgSize(self.mCurVersion)
	
	
	self.touchGroup = TouchGroup:create()
	self:addChild(self.touchGroup)
	
	local uiPanel = GUIReader:shareReader():widgetFromJsonFile("ui/Layout_Upgrade.json")
	local uiSize  = uiPanel:getSize()
	
	uiPanel:setPosition(ccp( (visibleSize.width-uiSize.width)/2, (visibleSize.height-uiSize.height)/2 ))
	self.touchGroup:addWidget(uiPanel)
	
	local lblText1 = ui.newTTFLabelWithShadow({text=GString["UpdateNotice1"], size=20, offset=2, color=G_COLORS[5], font=G_FONT_NAMES.HeiTi })
	local lblSize  = lblText1:getCascadeBoundingBox().size
	lblText1:setPosition((uiSize.width-lblSize.width)/2, uiSize.height/2+50)
	uiPanel:addNode(lblText1)
	
	local strtext = string.format("%s: %.2fM", GString["UpdateNotice2"], TotalPkgSize)
	local lblText2 = ui.newTTFLabelWithShadow({text=strtext, size=20, offset=2, color=G_COLORS[5], font=G_FONT_NAMES.HeiTi })
	lblSize  = lblText2:getCascadeBoundingBox().size
	lblText2:setPosition((uiSize.width-lblSize.width)/2, uiSize.height/2)
	uiPanel:addNode(lblText2)
	
	
	local tVerInfo = VerPropList[self.mCurVersion]
	
	local uiBtnUpdate = self.touchGroup:getWidgetByName("Btn_Update")
	uiBtnUpdate:addTouchEventListener(function(sender, evtype)
		if evtype==TOUCH_EVENT_BEGAN then
			
		elseif evtype==TOUCH_EVENT_ENDED then
			self.downloader:startUpdate(tVerInfo["version_file"])
			
			self.touchGroup:removeFromParent()
		elseif evtype==TOUCH_EVENT_CANCELED then
			
		end
	end)
end

--[[
	检查是否需要更新
]]
function UIHotUpdate:uiCheckUpdate(msgId, obj)
	local tVerInfo = VerPropList[self.mCurVersion]
	print("-----------check update-----------")
	print_r(tVerInfo)
	
	local showDialog = false
	if obj then
		showDialog = obj:getValue()
	end
	
	if tVerInfo then
		-- local fileLen = self.downloader:getFileLen(tVerInfo["version_file"])
		-- if fileLen>0 then
			if showDialog then
				self:uiShowUpdate()
			else
				self.downloader:startUpdate(tVerInfo["version_file"])
			end
		-- else
			-- print("mir_debug: file size is zero: ", tVerInfo["version_file"])
			-- self:uiUpdateFinish(false)
		-- end
	else
		self:uiUpdateFinish(false)
	end
end


function UIHotUpdate:uiUpdateFinish(isRestart)
	--从父层移除本层
	self:removeFromParent()
	--通知显示登陆界面
	CCNotificationCenter:sharedNotificationCenter():postNotification(G_NF_UPDATE_CHECK_FINISH, CCBool:create(isRestart))
end


function UIHotUpdate:loadVersion()
	self.mCurVersion = CCUserDefault:sharedUserDefault():getStringForKey("version")
end

function UIHotUpdate:saveVersion()
	CCUserDefault:sharedUserDefault():setStringForKey("version", self.mCurVersion);
	CCUserDefault:sharedUserDefault():flush();
end

--[[
	请求版本更新配置信息
]]
function RequestUpdateInfo()
	-- if not PlatformMgr.isNetworkEnabled() then
		-- PlatformMgr.makeToast(GString["NoNetwork"])
		-- return
	-- end
	
	-- VerPropList = {
		-- ["1.0"]={file="http://xxx/static/images/update.zip", pkg_size=2.06, new_version="1.01"}
	-- }
	local function onHttpResponse(event)
		local request = event.request
		local lresult = 0
		
		if event.name == "completed" then
			lresult = -1
			if request:getResponseStatusCode() == 200 then
				local jstr = request:getResponseString()
				local tlua = json.decode(jstr)
				
				if not tlua then
					jstr = string.match(jstr, "%[.*%]")
					tlua = json.decode(jstr)
				end
				
				if tlua then
					lresult = 1
					for k, tVerInfo in pairs(tlua) do
						if tVerInfo["ios_min_ver"] or tVerInfo["android_min_ver"] then
							
						else
							tVerInfo["version_file"] = string.urldecode(tVerInfo["version_file"])
							VerPropList[tVerInfo.version_id] = tVerInfo
						end
					end
					
					--获取获取更新文件成功
					CCNotificationCenter:sharedNotificationCenter():postNotification("NotifyRequestSucess", CCBool:create(true))
				end
			end
		elseif event.name=="inprogress" or event.name=="progress" then
		else	--android timeout or unkown host
			lresult = -1
		end
		if lresult == -1 then
			GameFunc.PerformWithDelay(0.1, function()
				PlatformMgr.switchHost("httpserver")
				RequestUpdateInfo()
			end)
		end
	end
	
	LoginNet.applyHttpRequest(PlatformMgr.url_update, 6, onHttpResponse)
	
end


function GetUpdatePkgSize()
	return TotalPkgSize
end

--[[
	设置下载文件包的大小
]]
function SetUpdatePkgSize(pkgSize)
	
end

--[[
	设置下载进度
]]
function SetUpdateProgress(nowDownloaded, totalDownload, percent, speed)
	-- print("====nowDownloaded, totalDownload, percent, speed=", nowDownloaded, totalDownload, percent, speed)
	
	local strinfo = string.format("正在更新%%%d(%d/%d %dkbs)_%f", percent, nowDownloaded, totalDownload, speed, percent)
	
	CCNotificationCenter:sharedNotificationCenter():postNotification(G_NF_UPDATE_PROGRESS_INFO, CCString:create(strinfo))
	
end

function IsContainVerInfo(version)
	if VerPropList[version] then
		return true
	end
	
	return false
end